"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.makeTestEnvironment = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _core = require("@resolve-js/core");

var _runtime = require("@resolve-js/runtime");

var _getSecretsManager = require("../../runtime/get-secrets-manager");

var _getEventStore = require("../../runtime/get-event-store");

var _getSagaRuntime = require("../../runtime/get-saga-runtime");

var _mockSideEffects = require("../../runtime/mock-side-effects");

var _getReadModelAdapter = require("../../runtime/get-read-model-adapter");

var _lodash = _interopRequireDefault(require("lodash.partial"));

var _assertions = require("../../utils/assertions");

var _utils = require("../../runtime/utils");

var promisedAssertion = function promisedAssertion(assertion, result, error) {
  return new Promise(function (resolve, reject) {
    return assertion(resolve, reject, result, error, false);
  });
};

var makeTestEnvironment = function makeTestEnvironment(context) {
  var mockedCommandImplementations = new Map();
  var mockedQueryImplementations = new Map();
  var executed = false;
  var useRealSideEffects = false;
  var sideEffectsStartTimestamp = 0;
  var assertions = [];
  var secretsManager = (0, _getSecretsManager.getSecretsManager)();
  var completeTest;
  var failTest;

  var setSecretsManager = function setSecretsManager(value) {
    secretsManager = value;
  };

  var allowSideEffects = function allowSideEffects() {
    useRealSideEffects = true;
  };

  var setSideEffectsStartTimestamp = function setSideEffectsStartTimestamp(value) {
    sideEffectsStartTimestamp = value;
  };

  var addAssertion = function addAssertion(value) {
    assertions.push((0, _lodash["default"])(promisedAssertion, value));
  };

  var isExecuted = function isExecuted() {
    return executed;
  };

  var mockCommandImplementation = function mockCommandImplementation(aggregateName, type, implementation) {
    mockedCommandImplementations.set((0, _utils.getCommandImplementationKey)({
      type: type,
      aggregateName: aggregateName
    }), implementation);
  };

  var mockQueryImplementation = function mockQueryImplementation(modelName, resolverName, implementation) {
    mockedQueryImplementations.set((0, _utils.getQueryImplementationKey)({
      modelName: modelName,
      resolverName: resolverName
    }), implementation);
  };

  var promise = new Promise(function (resolve, reject) {
    completeTest = resolve;
    failTest = reject;
  });

  var execute = /*#__PURE__*/function () {
    var _ref = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee4() {
      var result, saga, events, adapter, encryption, actualEncryption, actualAdapter, isNext, domain, liveErrors, monitoring, errors, executor, eventstoreAdapter, runtime, status, liveErrorIndex, _error2, summaryError;

      return _regenerator["default"].wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              executed = true;
              result = Object.defineProperty({
                commands: [],
                scheduledCommands: [],
                queries: [],
                sideEffects: [],
                scheduleCommands: []
              }, // FIXME: deprecated
              'scheduleCommands', {
                enumerable: false,
                get: function get() {
                  // eslint-disable-next-line no-console
                  console.warn("'scheduleCommands' property deprecated, use 'scheduledCommands' instead.");
                  return this.scheduledCommands;
                }
              });
              saga = context.saga, events = context.events, adapter = context.adapter, encryption = context.encryption;
              actualEncryption = encryption != null ? encryption : /*#__PURE__*/(0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee() {
                return _regenerator["default"].wrap(function _callee$(_context) {
                  while (1) {
                    switch (_context.prev = _context.next) {
                      case 0:
                        return _context.abrupt("return", null);

                      case 1:
                      case "end":
                        return _context.stop();
                    }
                  }
                }, _callee);
              }));

              if (!(adapter != null)) {
                _context4.next = 8;
                break;
              }

              _context4.t0 = adapter;
              _context4.next = 11;
              break;

            case 8:
              _context4.next = 10;
              return (0, _getReadModelAdapter.getReadModelAdapter)();

            case 10:
              _context4.t0 = _context4.sent;

            case 11:
              actualAdapter = _context4.t0;
              isNext = false;
              domain = (0, _core.initDomain)({
                viewModels: [],
                readModels: [],
                aggregates: [],
                sagas: [{
                  name: saga.name,
                  handlers: saga.handlers,
                  sideEffects: useRealSideEffects ? saga.sideEffects : (0, _mockSideEffects.mockSideEffects)(result, saga.sideEffects),
                  connectorName: 'ADAPTER_NAME',
                  encryption: actualEncryption
                }]
              });
              liveErrors = [];
              monitoring = {
                group: function group(config) {
                  if (config.Part !== 'ReadModelProjection') {
                    return monitoring;
                  }

                  return (0, _extends2["default"])({}, monitoring, {
                    error: function (_error) {
                      function error(_x) {
                        return _error.apply(this, arguments);
                      }

                      error.toString = function () {
                        return _error.toString();
                      };

                      return error;
                    }(function (error) {
                      liveErrors.push(error);
                    })
                  });
                },
                time: function time() {
                  return void 0;
                },
                timeEnd: function timeEnd() {
                  return void 0;
                },
                error: function error() {
                  return void 0;
                },
                publish: function () {
                  var _publish = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee2() {
                    return _regenerator["default"].wrap(function _callee2$(_context2) {
                      while (1) {
                        switch (_context2.prev = _context2.next) {
                          case 0:
                            return _context2.abrupt("return", void 0);

                          case 1:
                          case "end":
                            return _context2.stop();
                        }
                      }
                    }, _callee2);
                  }));

                  function publish() {
                    return _publish.apply(this, arguments);
                  }

                  return publish;
                }()
              };
              errors = [];
              executor = null;
              _context4.prev = 18;
              _context4.next = 21;
              return (0, _getEventStore.getEventStore)(events);

            case 21:
              eventstoreAdapter = _context4.sent;
              runtime = (0, _getSagaRuntime.getSagaRuntime)(result, {
                commands: mockedCommandImplementations,
                queries: mockedQueryImplementations
              }, domain.sagaDomain.schedulerName, secretsManager, monitoring, sideEffectsStartTimestamp);
              executor = (0, _runtime.createQuery)({
                applicationName: 'APP_NAME',
                readModelConnectors: {
                  ADAPTER_NAME: actualAdapter
                },
                getVacantTimeInMillis: function getVacantTimeInMillis() {
                  return 0x7fffffff;
                },
                invokeBuildAsync: function () {
                  var _invokeBuildAsync = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee3() {
                    return _regenerator["default"].wrap(function _callee3$(_context3) {
                      while (1) {
                        switch (_context3.prev = _context3.next) {
                          case 0:
                            isNext = true;

                          case 1:
                          case "end":
                            return _context3.stop();
                        }
                      }
                    }, _callee3);
                  }));

                  function invokeBuildAsync() {
                    return _invokeBuildAsync.apply(this, arguments);
                  }

                  return invokeBuildAsync;
                }(),
                eventstoreAdapter: eventstoreAdapter,
                readModelsInterop: domain.sagaDomain.acquireSagasInterop(runtime),
                viewModelsInterop: {},
                performanceTracer: null
              });
              _context4.prev = 24;
              _context4.next = 27;
              return eventstoreAdapter.ensureEventSubscriber({
                applicationName: 'APP_NAME',
                eventSubscriber: saga.name,
                status: null,
                destination: 'LOCAL'
              });

            case 27:
              _context4.next = 29;
              return executor.subscribe({
                modelName: saga.name,
                subscriptionOptions: {
                  eventTypes: null,
                  aggregateIds: null
                }
              });

            case 29:
              _context4.next = 31;
              return executor.resume({
                modelName: saga.name
              });

            case 31:
              isNext = false;
              _context4.next = 34;
              return executor.build({
                modelName: saga.name
              });

            case 34:
              if (isNext) {
                _context4.next = 31;
                break;
              }

            case 35:
              _context4.next = 37;
              return executor.status({
                modelName: saga.name
              });

            case 37:
              status = _context4.sent;

              if (!(status.status === 'error')) {
                _context4.next = 49;
                break;
              }

              if (Array.isArray(status.errors)) {
                _context4.next = 41;
                break;
              }

              throw new Error('Unknown error');

            case 41:
              liveErrorIndex = status.errors.length === 1 ? liveErrors.findIndex(function (err) {
                return err.message === status.errors[0].message;
              }) : -1;

              if (!(liveErrorIndex < 0)) {
                _context4.next = 48;
                break;
              }

              _error2 = new Error(status.errors.map(function (err) {
                return err.message;
              }).join('\n'));
              _error2.stack = status.errors.map(function (err) {
                return err.stack;
              }).join('\n');
              throw _error2;

            case 48:
              throw liveErrors[liveErrorIndex];

            case 49:
              _context4.next = 54;
              break;

            case 51:
              _context4.prev = 51;
              _context4.t1 = _context4["catch"](24);
              errors.push(_context4.t1);

            case 54:
              _context4.prev = 54;
              _context4.next = 57;
              return executor.unsubscribe({
                modelName: saga.name
              });

            case 57:
              _context4.next = 62;
              break;

            case 59:
              _context4.prev = 59;
              _context4.t2 = _context4["catch"](54);
              errors.push(_context4.t2);

            case 62:
              _context4.prev = 62;
              _context4.next = 65;
              return eventstoreAdapter.removeEventSubscriber({
                applicationName: 'APP_NAME',
                eventSubscriber: saga.name
              });

            case 65:
              _context4.next = 70;
              break;

            case 67:
              _context4.prev = 67;
              _context4.t3 = _context4["catch"](62);
              errors.push(_context4.t3);

            case 70:
              _context4.next = 75;
              break;

            case 72:
              _context4.prev = 72;
              _context4.t4 = _context4["catch"](18);
              errors.push(_context4.t4);

            case 75:
              _context4.prev = 75;

              if (!(executor != null)) {
                _context4.next = 85;
                break;
              }

              _context4.prev = 77;
              _context4.next = 80;
              return executor.dispose();

            case 80:
              _context4.next = 85;
              break;

            case 82:
              _context4.prev = 82;
              _context4.t5 = _context4["catch"](77);
              errors.push(_context4.t5);

            case 85:
              return _context4.finish(75);

            case 86:
              if (!(errors.length === 0)) {
                _context4.next = 102;
                break;
              }

              if (!(assertions.length > 0)) {
                _context4.next = 99;
                break;
              }

              _context4.prev = 88;
              _context4.next = 91;
              return Promise.all(assertions.map(function (assertion) {
                return assertion(result, null);
              }));

            case 91:
              completeTest(result);
              _context4.next = 97;
              break;

            case 94:
              _context4.prev = 94;
              _context4.t6 = _context4["catch"](88);
              failTest(_context4.t6);

            case 97:
              _context4.next = 100;
              break;

            case 99:
              (0, _assertions.defaultAssertion)(completeTest, failTest, result, null);

            case 100:
              _context4.next = 118;
              break;

            case 102:
              summaryError = errors[0];

              if (errors.length > 1) {
                summaryError = new Error(errors.map(function (err) {
                  return err.message;
                }).join('\n'));
                summaryError.stack = errors.map(function (err) {
                  return err.stack;
                }).join('\n');
              } // eslint-disable-next-line no-console


              console.error(summaryError);

              if (!(assertions.length > 0)) {
                _context4.next = 117;
                break;
              }

              _context4.prev = 106;
              _context4.next = 109;
              return Promise.all(assertions.map(function (assertion) {
                return assertion(result, errors[0]);
              }));

            case 109:
              failTest(errors[0]);
              _context4.next = 115;
              break;

            case 112:
              _context4.prev = 112;
              _context4.t7 = _context4["catch"](106);
              failTest(_context4.t7);

            case 115:
              _context4.next = 118;
              break;

            case 117:
              (0, _assertions.defaultAssertion)(completeTest, failTest, result, errors[0]);

            case 118:
            case "end":
              return _context4.stop();
          }
        }
      }, _callee4, null, [[18, 72, 75, 86], [24, 51], [54, 59], [62, 67], [77, 82], [88, 94], [106, 112]]);
    }));

    return function execute() {
      return _ref.apply(this, arguments);
    };
  }();

  setImmediate(execute);
  return {
    setSecretsManager: setSecretsManager,
    allowSideEffects: allowSideEffects,
    setSideEffectsStartTimestamp: setSideEffectsStartTimestamp,
    addAssertion: addAssertion,
    isExecuted: isExecuted,
    promise: promise,
    mockCommandImplementation: mockCommandImplementation,
    mockQueryImplementation: mockQueryImplementation
  };
};

exports.makeTestEnvironment = makeTestEnvironment;
//# sourceMappingURL=make-test-environment.js.map